/*****************************************************************************
* CINEDB.CPP																					  *
*																									  *
* Cinematronics Emulator / Debugger.													  *
*****************************************************************************/
#include	<stdio.h>
#include	<stdlib.h>
#include	<conio.h>
#include	<ctype.h>
#include	<malloc.h>
#include	<memory.h>
#include	<string.h>

#include	"zconio.h"

//#define	COUNT_VECTORS
//#define	LOG_IO_HITS

#define	VERSION	"Version 1.2"

#define	ADJTMR	0

#include	"cines.h"
#include	"inifile.h"
#include	"options.h"

#define	REFRSTD	62799U
#define	REFRFUL	0

#define	ONEVECT		10L		// number of bytes used for one vector
#define	ERAVECT		8L			// number of bytes to erasa one vector
#define	VECTORS		600L		// maximum number of vectors

#define	PROGSIZE		32768L
#define	VECTSIZE		(VECTORS * ONEVECT)	// max number of vectors
#define	ERASSIZE		(VECTORS * ERAVECT)	// Size of erase array
#define	SCRASIZE		129

// extension for configuration files

#define	INI		".ini"

uchar	far	*ProgDataM;	// malloc'd memory for CPU data
uchar	far	*ProgData;	// paragraph adjusted pointer to CPU data
uchar	far	*VectDataM;	// malloc'd memory for Vector Array Data
uchar	far	*VectData;	// paragraph adjusted pointer to Vector Array Data
uchar far	*Era1DataM;	// malloc'd memory for erasing old vectors
uchar far	*Era1Data;	// paragraph adjusted pointer to CPU data
uchar far	*Era2DataM;	// malloc'd memory for erasing old vectors
uchar far	*Era2Data;	// paragraph adjusted pointer to CPU data
char			*TempBfr;	// just a general use temperary buffer

uchar	far *kbFlags = (uchar far *)(0x00400017L);	// point to keyboard flags

/**************************** main **********************************/

#ifdef COUNT_VECTORS
extern uint	vectMax;
#endif

#ifdef LOG_IO_HITS

struct IOLOG_S
{	uint	addr;		// address of instruction
	uint	ioF;		// i/o flag; 0=inputs 1=switches 2=outputs
	uint	op;		// opcode / page; Upper Byte = page, Lower = opcode
};

IOLOG_S		*IOLog;
uint			MaxIO = 0;

#endif

void main( int argc, char *argv[])
{
	int	nameLen, err;

	//	FILE	*tfile;			//zzz-for dumping merged images

	static	char hdr[] =
	"\nCINEMU - Cinematronics Emulator. " VERSION
	"\n(c) Copyright 1997, Zonn Moore.  All rights reserved.\n";

	fputs( hdr, stderr);

	if (argc != 2)
	{	fputs( "\nUse: CINEMU config\n", stderr);
		fputs( "\nWhere:", stderr);
		fputs( "\n   config = Name of configuration file", stderr);
		fputs( " (an extension of \".INI\" is assumed).\n", stderr);
		exit( 1);
	}

	// allocate memory

	ProgDataM = (uchar far *)farmalloc( PROGSIZE + 16);	// prog data
	VectDataM = (uchar far *)farmalloc( VECTSIZE + 16);	// vector data
	Era1DataM = (uchar far *)farmalloc( ERASSIZE + 16);	// vector data
	Era2DataM = (uchar far *)farmalloc( ERASSIZE + 16);	// vector data
	TempBfr = (char *)malloc( SCRASIZE);						// scratch data

	if (ProgDataM == 0 || VectDataM == 0 || Era1DataM == 0 || Era2DataM == 0 ||
			TempBfr == 0)
	{	fputs( "\nNot enough memory!\n", stderr);
		exit( 0);
	}

#ifdef LOG_IO_HITS

	// Allocate some memory for I/O logging

	IOLog = (IOLOG_S *)malloc( sizeof( IOLOG_S) * 4096);

	for (int ii = 0; ii < 4096; ii++)
		IOLog[ii].ioF = 0xFF;

#endif

	/* Adjust to paragraph boundries */

	ProgData = (uchar far *)paraAdj( ProgDataM);
	VectData = (uchar far *)paraAdj( VectDataM);
	Era1Data = (uchar far *)paraAdj( Era1DataM);
	Era2Data = (uchar far *)paraAdj( Era2DataM);

	nameLen = strlen( argv[1]);				// get length of filename
	memcpy( TempBfr, argv[1], nameLen);		// get .INI's name

	if (TempBfr[nameLen-1] != '.' &&
			TempBfr[nameLen-2] != '.' &&
			TempBfr[nameLen-3] != '.' &&
			TempBfr[nameLen-4] != '.'
	)
		strcpy( TempBfr+nameLen, INI);			// add extension

	else
		TempBfr[nameLen] = '\0';

	err = openIniFile( TempBfr);

	if (err != iniErrOk)
		exit( err);

	err = execIniFile();

	if (err != iniErrOk)
		exit( err);

	// read ROMs

	if (OptRomImages == 0)
	{	fputs( "\nError: No ROM image files specified in configuration file.\n", stderr);
		exit( 1);
	}

	//zzz-"dump" clear memory

 //	for (unsigned int ll = 0; ll < PROGSIZE; ll++)
 //		ProgData[ll] = 0;

	if (readRoms( OptRomImages, ProgData, OptMemSize))
		exit( 1);

	//zzz-"dump" combined roms

 //	tfile = fopen( "dump.bin", "wb");
 //	fwrite( ProgData, 1, PROGSIZE, tfile);
 //	fclose( tfile);

	free( TempBfr);
	pComments();							// print all the printable comments
	closeIniFile();

	fputs( "\nPress a key to continue...", stderr);

	do
	{	getch();
	} while (kbhit());

	fputc( '\n', stderr);

	cineSetJMI( OptJMI);					// set JMI option
	cineSetMSize( OptMemSize);			// set memory size

	if (OptSpeed == 1)
		cineSetRate( REFRFUL);			// set to FULL speed

	else
		cineSetRate( REFRSTD);			// set to Standard rate

	cineSetSw( OptSwitches, OptInputs);			// set switches & inputs
	cineSetMonitor( OptMonitor);					// set up type of monitor
	cineSetMouse( OptMouseType, OptMouseSpeedX, OptMouseSpeedY,
			OptMouseValid, OptMKeyMap);	// set up mouse

	vgSetMode( 0x10);							// set to mode 10h - 640x350x16
	vgSetCineSize( OptWinXmin, OptWinYmin, OptWinXmax, OptWinYmax);

	vgSetMonitor( OptMonitor,				// set up type of monitor and colors
			OptRedB, OptGreenB, OptBlueB,
			OptRedC, OptGreenC, OptBlueC
	);

	vgSetTwinkle( OptTwinkle);			// set twinkle level
	vgSetRotateFlip( OptRotate, OptFlipX, OptFlipY);

	vgInit( VectData, Era1Data, Era2Data);	// setup video orientation and scaling

	cineInit( ProgData, OptKeyMap);	// setup segment pointer, reset breakpoints

	do
	{	cineReset();						// clear registers, initialize everything
	} while (cineExec());				// start emulator

	vsetmode( 0x03);						// set to mode 03h - 80x25x16

	// fix keyboard flags on the chance that someone used a shift or alt
	// sequence to continue into emulator.

	*kbFlags = 0;

	farfree( ProgDataM);
	farfree( VectDataM);
	farfree( Era1DataM);
	farfree( Era2DataM);

	fputs( hdr, stderr);

#ifdef COUNT_VECTORS
	printf( "\nVector Count = %u\n", vectMax);
#endif

// Display the log of I/O hits.

#ifdef LOG_IO_HITS
	uint	memSize;
	uint	romNP, rnameLen;

	if (OptMemSize == 1)
		memSize = 0x0800;

	else
		memSize = 0x1000;

	strupr( argv[1]);
	romNP = strlen( argv[1]);
	rnameLen = 0;

	while (--romNP > 0)
	{	rnameLen++;

		if (argv[1][romNP] == '\\')
		{	romNP++;
			rnameLen--;
			break;
		}
	}

	for (int jj = 0; jj < MaxIO; jj++)
	{
		// check if switch input, if so, point to 'SSA' instruction

		if (IOLog[jj].ioF == 1)
			IOLog[jj].addr--;

		fprintf( stdout, "\n%s.", argv[1] + romNP);

		if (OptMemSize != 0 && IOLog[jj].addr < memSize)
		{
			if (IOLog[jj].addr & 0x01)
				fputs( "P7", stdout);

			else
				fputs( "T7", stdout);
		}
		else
		{
			if (IOLog[jj].addr & 0x01)
				fputs( "R7", stdout);

			else
				fputs( "U7", stdout);
		}
		fprintf( stdout, "=%04X", (IOLog[jj].addr >> 1) & 0xFFF);

		if (IOLog[jj].ioF == 1)
		{
			IOLog[jj].addr++;				// point back to 'INP' instruction

			fprintf( stdout, ":57  [ssa   ]     %s.", argv[1] + romNP);

			if (OptMemSize != 0 && IOLog[jj].addr < memSize)
			{
				if (IOLog[jj].addr & 0x01)
					fputs( "P7", stdout);

				else
					fputs( "T7", stdout);
			}
			else
			{
				if (IOLog[jj].addr & 0x01)
					fputs( "R7", stdout);

				else
					fputs( "U7", stdout);
			}
			fprintf( stdout, "=%04X", (IOLog[jj].addr >> 1) & 0xFFF);
		}
//		else
//		{
//			for (int ii = 0; ii < rnameLen; ii++)
//				fputc( ' ', stdout);
	
//			fputs( "               ", stdout);
//		}

//		if (IOLog[jj].ioF == 1)
//			fputs( " SSA,", stdout);

		fprintf( stdout, ":%02X  ", IOLog[jj].op & 0xFF);

		if (IOLog[jj].ioF <= 1)
			fputs( "[inp", stdout);

		else
			fputs( "[out", stdout);

		fprintf( stdout, " %2u]", IOLog[jj].op & 0x0F);
	}
	fputc( '\n', stdout);
#endif
}
