;----------------------------------------------------------------------------;
; Routines for dealing with the Cinematronics Sound Boards.
;
; Author: Zonn Moore
; Date:   3/16/00
;
; (c) Copyright 2000, Zonn Moore.  All rights reserved.
;----------------------------------------------------------------------------;
		Ideal
		P386N

;MAXDACX		equ	1750
;MINDACX		equ	-1600
;
;MAXDACY		equ	1250
;MINDACY		equ	-1250

MAXDACX		equ	1536
MINDACX		equ	-1536

MAXDACY		equ	1536
MINDACY		equ	-1536

segment		_DATA	public 'DATA' dword
		extrn	JoyX:Word,JoyY:Word
		extrn	_ioInputs:Word,_ioOutputs:Byte
		extrn	MouseSpeedX:Word,MouseSpeedY:Word
		extrn	MouseType:Byte,OMouseKeys:Byte,VMouseKeys:Byte
		extrn	_OptSoundBdPort:Word
		ends

segment		_TEXT	public 'CODE' dword
		assume	cs:_TEXT
		extrn	keyPress:Near,keyRelease:Near

;----------------------------------------------------------------------------;
; Routine for setting a sound bit high.					     ;
;									     ;
; This routine is called by the OUT instruction of the C-CPU emulator	     ;
; core, when an output is set high.					     ;
;									     ;
; Called with:								     ;
;    AL = Bit-0 set.							     ;
;    CL = Bit number being set.						     ;
;    BL = Mask of bit being set.					     ;
;----------------------------------------------------------------------------;
public		soundSet
proc		soundSet	near

		mov	dx,[ds:_OptSoundBdPort]	; get address of port
		mov	al,[ds:_ioOutputs]	; get new value
		out	dx,al			; set new value
		
		cmp	[ds:MouseType],2	; Speed Freak?
		je	speedFrkSet
		cmp	[ds:MouseType],3	; Boxing Bug mouse?
		je	boxBugSet		; yes, deal with mouse
		ret

; *** Speed Freak ***

speedFrkSet:	cmp	cl,0			; bit zero being cleared?
		je	sfs1			; yes, clear mouse inputs
		ret

; only clear accumulator after a value has been read

sfs1:		test	[Byte ds:_ioInputs],00Fh ; anything set?
		jz	sfs2			 ; no, skip
		and	[Byte ds:_ioInputs],0F0h ; clear inputs
		mov	[ds:JoyX],0		 ; clear acc
		mov	[Byte ds:JoyX+2],0
sfs2:		ret

; *** Boxing Bugs ***

boxBugSet:	cmp	cl,1			; latch mouse?
		je	bbLatch
		cmp	cl,7			; switch to low nibble?
		je	bbLoNib
		ret

; Latch mouse

bbLatch:	test	[ds:_ioOutputs],080h	; high nibble?
		jz	bbHiNib1		; yes, return high nibble
		jmp	Short bbLoNib1

; Set low nibble to inputs of mouse latched

bbLoNib:	test	[ds:_ioOutputs],002h	; value latched?
		jz	bbLoNib2		; no, do nothing
bbLoNib1:	mov	ax,[ds:JoyX]		; get mouse position
		sar	ax,1			; scale and get low nibble
		and	[Byte ds:_ioInputs+1],00Fh ; remove old values
		or	[Byte ds:_ioInputs+1],al   ; add new values
bbLoNib2:	ret
		endp
		    
;----------------------------------------------------------------------------;
; Routine for setting a sound bit low.					     ;
;									     ;
; This routine is called by the OUT instruction of the C-CPU emulator	     ;
; core, when an output is set low.					     ;
;									     ;
; Called with:								     ;
;    AL = Bit-0 reset.							     ;
;    CL = Bit number being reset.					     ;
;    BL = Inverted mask of bit being reset.				     ;
;----------------------------------------------------------------------------;
public		soundReset
proc		soundReset	near

		mov	dx,[ds:_OptSoundBdPort]	; get address of port
		mov	al,[ds:_ioOutputs]	; get new value
		out	dx,al			; set new value
		
		cmp	[ds:MouseType],2	; Speed Freak?
		je	speedFrkRes
		cmp	[ds:MouseType],3	; Boxing Bug mouse?
		je	boxBugRes		; yes, deal with mouse
		ret

; *** Speed Freak ***

speedFrkRes:	cmp	cl,0			; bit zero being cleared?
		je	sfr1			; yes, clear mouse inputs
		ret

; only clear accumulator after a value has been read

sfr1:		test	[Byte ds:_ioInputs],00Fh ; anything set?
		jz	sfr2			 ; no, skip
		and	[Byte ds:_ioInputs],0F0h ; clear inputs
		mov	[ds:JoyX],0		 ; clear acc
		mov	[Byte ds:JoyX+2],0
sfr2:		ret

; *** Boxing Bugs ***

boxBugRes:	cmp	cl,1			; unlatch mouse?
		je	bbUnLatch
		cmp	cl,7			; switch to high nibble?
		je	bbHiNib
		ret

; Unlatch mouse

bbUnLatch:	and	[Byte ds:_ioInputs+1],00Fh ; zero mouse values
		ret

; Set low nibble to inputs of mouse latched

bbHiNib:	test	[ds:_ioOutputs],002h	; value latched?
		jz	bbHiNib2		; no, do nothing
bbHiNib1:	mov	ax,[ds:JoyX]		; get mouse position
		sar	ax,5			; scale value
		and	al,0F0h			; get high nibble
		and	[Byte ds:_ioInputs+1],00Fh ; remove old values
		or	[Byte ds:_ioInputs+1],al   ; add new values
bbHiNib2:	ret
		endp

;----------------------------------------------------------------------------;
; soundIO								     ;
;									     ;
; Called to do whatever I/O might be needed by this sound card.		     ;
; This is where Mouse and Joystick stuff is done.			     ;
;									     ;
; This is where sound table calculations would probably be done.	     ;
;----------------------------------------------------------------------------;
public		soundIO
proc		soundIO		near
		cmp	[ds:MouseType],0	; mouse enabled?
		jne	mouse			; yes, read position
		ret

; Read mouse button presses

mouse:		push	ds
		mov	ax,3			; get button info
		int	33h			; call mouse driver
		pop	ds

		mov	al,bl
		mov	ah,al			; save current values
		xor	al,[ds:OMouseKeys]	; check for changed keys
		mov	[ds:OMouseKeys],ah	; save new key settings
		and	al,[ds:VMouseKeys]	; check for valid keys

		; check button #1

		sub	bx,bx
		test	al,001h			; key changed states?
		jz	km2			; no, skip
		test	ah,001h			; key now pressed?
		jz	km1			; no, key released
		call	keyPress		; yes, do key press
		jmp	Short km2

km1:		call	keyRelease		; else, do key release
km2:

		; check button #2

		inc	bx
		test	al,002h			; key changed states?
		jz	km4			; no, skip
		test	ah,002h			; key now pressed?
		jz	km3			; no, key released
		call	keyPress		; yes, do key press
		jmp	Short km4

km3:		call	keyRelease		; else, do key release
km4:

		; check button #3

		inc	bx
		test	al,004h			; key changed states?
		jz	km6			; no, skip
		test	ah,004h			; key now pressed?
		jz	km5			; no, key released
		call	keyPress		; yes, do key press
		jmp	Short km6

km5:		call	keyRelease		; else, do key release
km6:

; Read mickeys and convert to position

		push	ds
		mov	ax,11			; read mickeys
		int	33h			; call mouse driver
		pop	ds

		cmp	[ds:MouseType],1	; Tailgunner?
		je	msTG
		cmp	[ds:MouseType],2	; Speed Freak?
		je	msSF
		cmp	[ds:MouseType],3	; Boxing Bugs?
		je	msBB
		ret

; *** Speed Freak Mouse Handler ***

msSF:		mov	ax,cx			; get X value
		imul	[ds:MouseSpeedX]	; scale
		mov	dh,dl			; get integer part
		mov	dl,ah
		add	[Byte ds:JoyX+2],al	; add fractional part
		adc	[ds:JoyX],dx		; add integer part

		test	[ds:_ioOutputs],001h	; mouse reset?
		jnz	msSFend			; yes, nothing to do

		mov	cx,[ds:JoyX]		; get current value of steering
		mov	al,[Byte ds:_ioInputs]	; get current inputs
		shr	al,1			; remove sign
		sub	ah,ah			; indicate positive
		test	cx,cx			; mouse negative?
		jns	msSFP			; jump if positive
		mov	ah,[Byte ds:_ioInputs]	; get fractional part
		not	cx			; negate
		neg	ah			; include frac in negate
		sbb	cx,0			; adjust high word
		mov	ah,1			; indicate negative
msSFP:		shr	cx,6			; scale down mouse
		jz	msSFend
		inc	cl
		shl	al,cl			; shift in zeros
		or	al,ah
		not	al			; invert bits
		and	al,00Fh			; clear high bits
		and	[Byte ds:_ioInputs],0F0h ; clear old values
		or	[Byte ds:_ioInputs],al	; set new value
msSFend:	ret

; *** Boxing Bugs Mouse Handler

		; Scale X value

msBB:		mov	ax,cx			; get X value
		imul	[ds:MouseSpeedX]	; scale
		mov	dh,dl			; get integer part
		mov	dl,ah
		add	[Byte ds:JoyX+2],al	; add fractional part
		adc	[ds:JoyX],dx		; add integer part

		; Test if mouse outputs are enable

		and	[Byte ds:_ioInputs+1],00Fh ; zero mouse inputs
		test	[ds:_ioOutputs],002h	; if high, outputs latched
		jnz	msBB1			; get proper nibble
		ret
 
msBB1:		mov	ax,[ds:JoyX]		; get mouse pointer
		sar	ax,5			; scale it down a bit
		test	[ds:_ioOutputs],080h	; output 7 used to mux nibbles
		jz	msBB2			; if low, use high nibble
		shl	al,4			; else, use low nibble
msBB2:		and	al,0F0h			; keep only high bits
		or	[Byte ds:_ioInputs+1],al ; set new mouse inputs
		ret

; *** Tailgunner mouse handler ***

		; scale Y axis

msTG:		neg	dx			; invert Y axis
		mov	ax,dx			; get Y value
		imul	[ds:MouseSpeedY]	; scale
		mov	dh,dl			; get integer part
		mov	dl,ah
		add	[Byte ds:JoyY+2],al	; add fractional part
		adc	dx,[ds:JoyY]		; add integer part

		cmp	dx,MAXDACY		; greater than max?
		jle	kp1			; no, then skip
		mov	dx,MAXDACY		; else, peg it
		mov	[Byte ds:JoyY+2],0	; zero fractional part
		jmp	Short kp2

kp1:		cmp	dx,MINDACY		; less than min?
		jge	kp2			; ok, skip
		mov	dx,MINDACY		; peg it
		mov	[Byte ds:JoyY+2],0FFh	; zero fractional part
kp2:		mov	[ds:JoyY],dx		; save new position

		; scale X axis

		mov	ax,cx			; get X value
		imul	[ds:MouseSpeedX]	; scale
		mov	dh,dl			; get integer part
		mov	dl,ah
		add	[Byte ds:JoyX+2],al	; add fractional part
		adc	dx,[ds:JoyX]		; add integer part

		cmp	dx,MAXDACX		; greater than max?
		jle	kp3			; no, then skip
		mov	dx,MAXDACX		; else, peg it
		mov	[Byte ds:JoyX+2],0	; zero fractional part
		jmp	Short kp4

kp3:		cmp	dx,MINDACX		; less than min?
		jge	kp4			; ok, skip
		mov	dx,MINDACX		; peg it
		mov	[Byte ds:JoyX+2],0FFh	; zero fractional part
kp4:		mov	[ds:JoyX],dx		; save new position
		ret
		endp

		ends
		end
