;----------------------------------------------------------------------------;
; Vector Generator based on Cinematronics hardware.
;
; Author: Zonn Moore
; Date:   3/16/00
;
; (c) Copyright 2000, Zonn Moore.  All rights reserved.
;----------------------------------------------------------------------------;
		Ideal
		P386N
		include	"zvga.inc"
		include	"video.inc"
		include	"v64b4.inc"

CLRSCN		equ	1		; 1=Clear screen, 0=Erase vectors

GRAPHICS	equ	0		; if defined, print graphics
					; else, no screen updates (for test purposes)

NEWX		equ	410

;NOSWAP		equ	0
;COUNT_VECTORS	equ	0


VECTMEMSZ	equ	10		; size of one vector in array
ERASMEMSZ	equ	8		; size of one erase vector in array

segment		_DATA	public 'DATA' dword
		extrn	_ioOutputs:Byte,vCurPge:Word,vCurOfs:Word

 ifdef DEBUGGER
		extrn	_insCount:word

public		_vgXstart,_vgXend,_vgYstart,_vgYend,_vgRange,_vgColor
 endif

 ifdef NOSWAP
zzXstart	dw	?
zzXend		dw	?
zzYstart	dw	?
zzYend		dw	?
zzRange		db	?
 endif

public		vgVecPtr
label		vgVecPtr	Dword	; Full pointer
vgVecOfs	dw	?		; Vector pointer Offset
vgVecSeg	dw	?		; Vector pointer Segment

label		vgCurEraPtr	Dword	; Full pointer to current erase buffer
vgCurEraOfs	dw	?		; Current erase Offset
vgCurEraSeg	dw	?		; Current erase Segment

label		vgLastEraPtr	Dword	; Full pointer to last erase buffer
vgLastEraOfs	dw	?		; Last erase Offset
vgLastEraSeg	dw	?		; Last erase Segment

vgVgaXScale	dw	?		; scale to convert to X VGA
vgVgaXMulOne	dw	?		; set to FFFF if multiply .gt. one
vgVgaXOffset	dw	?		; X offset / inversion
vgVgaXSignFix	dw	?		; X correction for negation

vgVgaYScale	dw	?		; scale to convert to Y VGA
vgVgaYMulOne	dw	?		; set to FFFF if multiply .gt. one
vgVgaYOffset	dw	?		; Y offset / inversion
vgVgaYSignFix	dw	?		; Y correction for negation

vgNewVGAX	dw	?		; X changes if rotated

vgCineX		dw	?		; X pixels of current Cine' size
vgCineY		dw	?		; Y pixels of current Cine' size

vgCineXmin	dw	?		; Smallest X value of Cine' screen
vgCineYmin	dw	?		; Smallest Y value of Cine' screen
vgCineXmax	dw	?		; Largest X value of Cine' screen
vgCineYmax	dw	?		; Largest Y value of Cine' screen

vgColor		dw	?		; Color word
vgTwinkle	db	?		; Twinkle level

vgRotate	db	?		; set if X and Y need swapping
vgFlipX		db	?		; set if X needs flipping
vgFlipY		db	?		; set if Y needs flipping

vgTmpColor	dw	?		; Temp color word

 ifdef DEBUGGER
_vgXstart	dw	?
_vgXend		dw	?

_vgYstart	dw	?
_vgYend		dw	?

_vgRange	db	?
_vgColor	db	?
 endif

 ifdef COUNT_VECTORS
public		_vectCount,_vectMax
_vectCount	dw	?
_vectMax	dw	?
 endif

		ends

segment		_TEXT	public 'CODE' dword
		assume	cs:_TEXT

		extrn	mouseHandler:Near

		assume	ds:nothing,fs:nothing,es:nothing,gs:nothing

;----------------------------------------------------------------------------;
; vgSetCineSize( int aCinXmin, int aCinYmin, int aCinXmax, int aCinYmax)     ;
;									     ;
; Set the size of the Cinematronics window.				     ;
;									     ;
; No checks are done to verify data.  Invalid ranges can lead to divide	     ;
; errors further on down the road...					     ;
;----------------------------------------------------------------------------;
public		_vgSetCineSize
proc		_vgSetCineSize c near
		arg	aXmin:word,aYmin:word,aXmax:word,aYmax:word

		mov	ax,[aXmax]		; get X max
		mov	[ds:vgCineXmax],ax	; save it
		mov	bx,[aXmin]		; get X min
		mov	[ds:vgCineXmin],bx	; save it
		sub	ax,bx			; get length of line
		inc	ax			; get number of pixels
		mov	[ds:vgCineX],ax		; save it

		mov	ax,[aYmax]		; get Y max
		mov	[ds:vgCineYmax],ax	; save it
		mov	bx,[aYmin]		; get Y min
		mov	[ds:vgCineYmin],bx	; save it
		sub	ax,bx			; get length of line
		inc	ax			; get number of pixels
		mov	[ds:vgCineY],ax		; save it
		ret
		endp

;----------------------------------------------------------------------------;
; vgSetTwinkle( int Twinkle)						     ;
;									     ;
; Set the Twinkle level.						     ;
;----------------------------------------------------------------------------;
public		_vgSetTwinkle
proc		_vgSetTwinkle	c near
		arg	aTwinkle:word

		mov	al,[Byte aTwinkle]
		mov	[ds:vgTwinkle],al
		ret
		endp

;----------------------------------------------------------------------------;
; vgSetRotateFlip( int rotate, int flipX, int flipY)			     ;
;									     ;
; Set flip and rotate flags.  Rotation will be done first, then flips.	     ;
; The rotation and flips are not done in this routine, but are done when     ;
; a call to vgInit is executed.						     ;
;----------------------------------------------------------------------------;
public		_vgSetRotateFlip
proc		_vgSetRotateFlip c near
		arg	rotate:word,flipX:word,flipY:word

		mov	ax,[rotate]		; get rotate flag
		or	al,ah			; make 1 byte
		mov	[ds:vgRotate],al	; save flag

		mov	ax,[flipX]		; get flip X flag
		or	al,ah			; make 1 byte
		mov	[ds:vgFlipX],al		; save flag

		mov	ax,[flipY]		; get flip Y flag
		or	al,ah			; make 1 byte
		mov	[ds:vgFlipY],al		; save flag
		ret
		endp

;----------------------------------------------------------------------------;
; vgInit( uchar far *vectBfr)						     ;
;									     ;
; Initialize vector generator.						     ;
;									     ;
; VGA size (mode), Cine' size, Rotate and Flip must all be setup before	     ;
; this call.								     ;
;----------------------------------------------------------------------------;
public		_vgSetMode
proc		_vgSetMode c near
		call	vInit

 ifdef COUNT_VECTORS
		mov	[ds:_vectCount],0
		mov	[ds:_vectMax],0
 endif

		ret
		endp

public		_vgInit
proc		_vgInit c near
		arg	vectBfr:dword,era1Bfr:dword,era2Bfr:dword

		mov	ax,[Word vectBfr+2]	; get buffers segment
		mov	[ds:vgVecSeg],ax	; save Vector segment
		mov	[ds:vgVecOfs],0		; zero pointer

		mov	ax,[Word era1Bfr+2]
		mov	[ds:vgCurEraSeg],ax	; save segment
		mov	[ds:vgCurEraOfs],0	; zero pointer

		mov	ax,[Word era2Bfr+2]
		mov	[ds:vgLastEraSeg],ax	; save segment
		mov	[ds:vgLastEraOfs],0	; zero pointer

		mov	ax,[ds:vXSize]		; get X size
		mov	[ds:vgNewVGAX],ax	; save non-rotated value

; Check if rotated

		cmp	[ds:vgRotate],0		; rotate?
		jnz	vgIniRotate		; yes, do rotate calc's

; Else setup for standard screen

; vgVgaXScale = (vgVGAX * 65536) / CineX
; vgVgaXOffset = 0
; vgVgaXSignFix = 0
;
; vgVgaYScale = (vgVGAY * 65536) / CineY
; vgVgaYOffset = -vgVGAY	; pre-flip Y image
; vgVgaYSignFix = -1

		mov	dx,[ds:vXSize]		; get X size
		mov	[ds:vgNewVGAX],dx	; save non-rotated value
		mov	[ds:vgVgaXMulOne],0	; zero '1' multiplier
		cmp	dx,[ds:vgCineX]		; result will be .lt. 1?
		jb	vgi1			; yes, skip
		mov	[ds:vgVgaXMulOne],0FFFFh	; else, set '1' multiplier
		sub	dx,[ds:vgCineX]		; get only fractional part
vgi1:		sub	ax,ax			; mult by 65536
		div	[ds:vgCineX]		; divide by CineX
		mov	[ds:vgVgaXScale],ax	; save scale
		mov	[ds:vgVgaXOffset],0	; zero offset
		mov	[ds:vgVgaXSignFix],0	; zero sign

		mov	dx,[ds:vYSize]		; get Y size
		mov	[ds:vgVgaYMulOne],0	; zero '1' multiplier
		cmp	dx,[ds:vgCineY]		; result will be .lt. 1?
		jb	vgi2			; yes, skip
		mov	[ds:vgVgaYMulOne],0FFFFh	; else, set '1' multiplier
		sub	dx,[ds:vgCineY]		; get only fractional part
vgi2:		sub	ax,ax			; mult by 65536
		div	[ds:vgCineY]		; divide by CineY
		mov	[ds:vgVgaYScale],ax	; save scale

		mov	ax,[ds:vYSize]		; get Y pixels
		neg	ax			; invert
		mov	[ds:vgVgaYOffset],ax	; set offset
		mov	[ds:vgVgaYSignFix],-1	; set sign fix to invert
		jmp	vgIniFlipX		; check to flip X

; Else setup for rotated screen

; vgVgaYScale = (vgVGAY * 65536) / CineX
; vgVgaYOffset = 0
; vgVgaYSignFix = 0
;
; vgVgaXScale = (NEWX * 65536) / CineY				; for now...
; vgVgaXOffset = (vgVGAX - (vgCineY * vgVgaXScale)) / 2
; vgVgaXSignFix = 0

vgIniRotate:	mov	dx,[ds:vYSize]		; get Y size
		sub	ax,ax			; mult by 65536
		div	[ds:vgCineX]		; divide by CineX
		mov	[ds:vgVgaYScale],ax	; save scale
		mov	[ds:vgVgaYOffset],0	; zero offset
		mov	[ds:vgVgaYSignFix],0	; zero sign

		; get X scale

		mov	dx,NEWX			; get rotated value
		mov	[ds:vgNewVGAX],dx	; change rotated X value
		sub	ax,ax			; zero lower value
		div	[ds:vgCineY]		; calc X scale
		mov	[ds:vgVgaXScale],ax	; save scale

		; calculate offset

		mov	ax,[ds:vgCineY]		; get Cine Y length
		mul	[ds:vgVgaXScale]	; get length of Cine Y
		neg	dx			; invert it
		add	dx,[ds:vXSize]		; subtract it from full screen
		shr	dx,1			; divide by two for offset
		mov	[ds:vgVgaXOffset],dx	; save offset
		mov	[ds:vgVgaXSignFix],0	; set sign fix to invert

; Check to see if X needs flipping

vgIniFlipX:	cmp	[ds:vgFlipX],0		; flip X-axis?
		jz	vgIniFlipY		; no, check Y-axis

		mov	ax,[ds:vgVgaXOffset]	; get current offset
		add	ax,[ds:vgNewVGAX]	; new screen width
		neg	ax			; invert it
		mov	[ds:vgVgaXOffset],ax	; save new value
		not	[ds:vgVgaXSignFix]	; invert sign fix

; Check to see if Y needs flipping

vgIniFlipY:	cmp	[ds:vgFlipY],0		; flip Y-axis?
		jz	vgIni1			; no, done

		mov	ax,[ds:vgVgaYOffset]	; get current offset
		add	ax,[ds:vYSize]		; add screen width
		neg	ax			; invert it
		mov	[ds:vgVgaYOffset],ax	; save new value
		not	[ds:vgVgaYSignFix]	; invert sign fix

vgIni1:		ret
		endp

; Called with AX containing percentage.

proc		calcPercRGB
		mul	[cs:maxRGB]		; get percentage of RGB
		add	ax,50			; do 5/4 rounding
		div	[cs:hundred]
		ret
		endp

public		_vgSetMonitor
proc		_vgSetMonitor	c near
		arg	aMonitor:word
		arg	aRedB:word,aGreenB:word,aBlueB:word
		arg	aRedC:word,aGreenC:word,aBlueC:word
		uses	si

		local	redIncL:word,redIncH:word
		local	greenIncL:word,greenIncH:word
		local	blueIncL:word,blueIncH:word
		local	redH:word,redL:word
		local	greenH:word,greenL:word
		local	blueH:word,blueL:word
		
		cmp	[aMonitor],1		; 16-Level?
		je	vgSet16			; yes, set for 16-level
		cmp	[aMonitor],2		; 64-Level?
		je	vgSet64			; yes, set for 64-level
		cmp	[aMonitor],3		; Color?
		je	vgSetCo			; yes, set for color

		; Assume Bi-Level

		mov	ax,[aBlueB]		; get blue's brightness
		call	calcPercRGB
		mov	cl,al			; save Blue
		mov	ax,[aGreenB]
		call	calcPercRGB
		mov	ch,al			; save Green
		mov	ax,[aRedB]
		call	calcPercRGB
		mov	ah,al			; save Red
		mov	al,03Fh			; Brightness color reg
		call	vgMapColor		; map brightness

		mov	ax,[aBlueC]		; get blue's contrast
		call	calcPercRGB
		mov	cl,al			; save Blue
		mov	ax,[aGreenC]
		call	calcPercRGB
		mov	ch,al			; save Green
		mov	ax,[aRedC]
		call	calcPercRGB
		mov	ah,al			; save Red
		mov	al,007h			; Contrast color reg
		call	vgMapColor		; map contrast
		ret				; do nothing for Bi-Level

vgSet64:
vgSet16:
	 	mov	ax,[aRedC]		; get low value
		call	calcPercRGB
		mov	[redH],ax		; save intial value
		mov	[redL],08000h		; do 5/4 rounding
		push	ax
		mov	ax,[aRedB]		; get high value
		call	calcPercRGB
		pop	dx
		call	calcInc			; get increment
		mov	[redIncH],dx		; save integer part
		mov	[redIncL],ax		; save fractional part

	 	mov	ax,[aGreenC]		; get low value
		call	calcPercRGB
		mov	[greenH],ax		; save intial value
		mov	[greenL],08000h		; do 5/4 rounding
		push	ax
		mov	ax,[aGreenB]		; get high value
		call	calcPercRGB
		pop	dx
		call	calcInc			; get increment
		mov	[greenIncH],dx		; save integer part
		mov	[greenIncL],ax		; save fractional part

	 	mov	ax,[aBlueC]		; get low value
		call	calcPercRGB
		mov	[blueH],ax		; save intial value
		mov	[blueL],08000h		; do 5/4 rounding
		push	ax
		mov	ax,[aBlueB]		; get high value
		call	calcPercRGB
		pop	dx
		call	calcInc			; get increment
		mov	[blueIncH],dx		; save integer part
		mov	[blueIncL],ax		; save fractional part

		mov	bx,Offset paletteTbl
		mov	cl,[Byte blueH]		; get starting blue
		mov	ch,[Byte greenH]	; get starting green
		mov	ah,[Byte redH]		; get starting red
		mov	al,[cs:bx]		; get starting color
		call	vgMapColor		; map color
		inc	bx			; point to next color

vgSet16L1:	mov	ax,[blueIncL]		; get fractional part
		add	[blueL],ax		; add
		mov	ax,[blueH]		; get integer part
		adc	ax,[blueIncH]		; add
		mov	[blueH],ax		; save new value
		mov	cl,al			; save BLUE value

		mov	ax,[greenIncL]		; get fractional part
		add	[greenL],ax		; add
		mov	ax,[greenH]		; get integer part
		adc	ax,[greenIncH]		; add
		mov	[greenH],ax		; save new value
		mov	ch,al			; save GREEN value

		mov	ax,[redIncL]		; get fractional part
		add	[redL],ax		; add
		mov	ax,[redH]		; get integer part
		adc	ax,[redIncH]		; add
		mov	[redH],ax		; save new value
		mov	ah,al			; save RED value

		mov	al,[cs:bx]		; get color
		call	vgMapColor		; set color
		inc	bx			; point to next color
		cmp	bx,Offset paletteEnd	; done?
		jb	vgSet16L1		; no, loop
		ret

vgSetCo:	mov	si,Offset colorReduct
		mov	bx,Offset paletteTbl

vgSetCoL1:	sub	ah,ah			; zero high byte
		mov	al,[cs:si]		; get blue
		inc	si
		call	calcPercRGB
		mov	cl,al
		mov	al,[cs:si]		; get green
		inc	si
		call	calcPercRGB
		mov	ch,al
		mov	al,[cs:si]		; get red
		call	calcPercRGB
		mov	ah,al
		inc	si
		mov	al,[cs:bx]		; get color
		call	vgMapColor		; set color
		inc	bx			; point to next color
		cmp	bx,Offset paletteEnd	; done?
		jb	vgSetCoL1		; no, loop
		ret

; Mappings of the Default VGA palette

label		paletteTbl	byte

		db	1h,2h,3h,4h,5h,14h,7h
		db	38h,39h,3ah,3bh,3ch,3dh,3eh,3fh

label		paletteEnd	byte

hundred		dw	100			; percentage divisor
maxRGB		dw	63			; max RGB value
intervals16	dw	14			; number of intervals of 16

label		colorReduct	byte

		; blue, green, red

		db	0,0,50
		db	0,50,0
		db	0,50,50
		db	50,0,0
		db	50,0,50
		db	50,50,0
		db	50,50,50

		db	0,0,100
		db	0,100,0
		db	0,100,100
		db	100,0,0
		db	100,0,100
		db	100,100,0
		db	100,100,100

		db	33,33,33		; dwhite

		endp

;----------------------------------------------------------------------------;
; Map an RGB value into a color.					     ;
;									     ;
; Called with								     ;
;    AH = Red   (0-63)							     ;
;    CH = Green (0-63)							     ;
;    CL = Blue  (0-63)							     ;
;    AL = Address of color						     ;
;----------------------------------------------------------------------------;
proc		vgMapColor	near
		mov	dx,VGAPELW	; point to PEL write register
		out	dx,al		; point to proper color register
		inc	dx		; point to data register
		mov	al,ah		; get RED
		out	dx,al		; set RED value
		IOWAIT
		IOWAIT
		IOWAIT
		IOWAIT
		mov	al,ch		; get GREEN
		out	dx,al		; set GREEN value
		IOWAIT
		IOWAIT
		IOWAIT
		IOWAIT
		mov	al,cl		; get BLUE value
		out	dx,al		; set BLUE value
		IOWAIT
		IOWAIT
		IOWAIT
		IOWAIT
		ret
		endp

;----------------------------------------------------------------------------;
; vgBrightBI								     ;
;									     ;
; Set brightness value based on a Bi-Level monitor.			     ;
;									     ;
; Called with CL - Bit0 with value of Brightness Bit			     ;
;----------------------------------------------------------------------------;
		align	4
public		vgBrightBI
proc		vgBrightBI	Near

		call	[ds:vXltBi]	; translate Bi-level color
		mov	[ds:vgColor],bx	; save new color
		ret
		endp

;----------------------------------------------------------------------------;
; vgBright16								     ;
;									     ;
; Set brightness value based on a 16-Level monitor.			     ;
;									     ;
; Level saved in most current 'VIN' instruction's A-reg as:		     ;
;									     ;
;    xxxx xxxx xxxx CCCC						     ;
;									     ;
; Where:								     ;
;    CCCC = Intensity level.						     ;
;           0000 = Lowest intensity					     ;
;           0001 = 2nd lowest intensity					     ;
;           1111 = Highest intensity					     ;
;----------------------------------------------------------------------------;
		align	4
public		vgBright16
proc		vgBright16	Near

		les	bx,[ds:vgVecPtr]	; point to vector array
		mov	bx,[es:bx]		; get color
		call	[ds:vXlt16]		; translate color
		jz	vg16L1			; no latch, skip
		mov	[ds:vgColor],bx		; save color
vg16L1:		ret
		endp

;----------------------------------------------------------------------------;
; vgBright64								     ;
;									     ;
; Set brightness value based on a 64-Level monitor.			     ;
;									     ;
; Level saved in most current 'VIN' instruction's A-reg as:		     ;
;									     ;
;    xxxx xxxx CCCC CCxx						     ;
;									     ;
; Where:								     ;
;    CCCC CC = Intensity level.						     ;
;              1111 11 = Lowest intensity				     ;
;              1111 10 = 2nd lowest intensity				     ;
;              0000 00 = Highest intensity				     ;
;----------------------------------------------------------------------------;
		align	4
public		vgBright64
proc		vgBright64	Near
		les	bx,[ds:vgVecPtr]	; point to vector array
		mov	bx,[es:bx]		; get color
		call	[ds:vXlt64]		; translate color
		jz	vg64L1			; no latch, skip
		mov	[ds:vgColor],bx		; save color
vg64L1:		ret
		endp

;----------------------------------------------------------------------------;
; vgBrightCo								     ;
;									     ;
; Set brightness value based on WotW color.				     ;
;									     ;
; Level saved in most current 'VIN' instruction's A-reg as:		     ;
;									     ;
;    xxxx BBBB GGGG RRRR						     ;
;									     ;
; Where:								     ;
;    BBBB = Blue intensity level.					     ;
;    GGGG = Green intensity level.					     ;
;    RRRR = Red intensity level.					     ;
;									     ;
;    1111 1111 1111 = Lowest intensity white.				     ;
;    1110 1111 1111 = Lowest intensity blue.				     ;
;    1111 1110 1111 = Lowest intensity green.				     ;
;    1111 1111 1110 = Lowest intensity red.				     ;
;    0000 0000 0000 = Brightest white.					     ;
;----------------------------------------------------------------------------;
		align	4
public		vgBrightCo
proc		vgBrightCo	Near

		les	bx,[ds:vgVecPtr]	; point to vector array
		mov	bx,[es:bx]		; get color
		call	[ds:vXltColor]		; translate color
		jz	vgWWL1			; no latch, skip
		mov	[ds:vgColor],bx		; save color
vgWWL1:         ret
		endp

;----------------------------------------------------------------------------;
; vgStart								     ;
;									     ;
; Set the starting address of a vector.					     ;
;									     ;
; Called with:								     ;
;    AX = X start							     ;
;    DX = Y start							     ;
;----------------------------------------------------------------------------;
		align	4
public		vgStart
proc		vgStart		Near

 ifdef NOSWAP
		mov	[ds:zzXstart],ax	; save X start
		mov	[ds:zzYstart],dx	; save Y start
 else

		lds	bx,[ds:vgVecPtr]	; point to vector array
		mov	[ds:bx],ax		; save 'vgXstart'
		shl	dx,4			; shift 'vgYstart'
		mov	[ds:bx+2],dx		; save 'vgYstart'
		shr	dx,4			; fix
 endif

		ret
		endp

;----------------------------------------------------------------------------;
; vgLength								     ;
;									     ;
; Set the length of a vector.						     ;
;									     ;
; Called with:								     ;
;    BL = Number of shifts loaded into length register			     ;
;----------------------------------------------------------------------------;
		align	4
public		vgLength
proc		vgLength	Near

 ifdef NOSWAP
		mov	[ds:zzRange],bl
 else
		lds	di,[ds:vgVecPtr]	; point to vector array
		or	[ds:di+2],bl		; save shift value
 endif

		ret
		endp

;----------------------------------------------------------------------------;
; vgDraw								     ;
;									     ;
; Set ending points and draw a vector, or save in buffer for later drawing.  ;
;									     ;
; Called with:								     ;
;    AX = X end								     ;
;    DX = Y end								     ;
;----------------------------------------------------------------------------;
		align	4
public		vgDraw
proc		vgDraw	near
		les	di,[ds:vgVecPtr]	; point to vector array
		mov	[es:di+4],ax		; save 'vgXend'
		mov	[es:di+6],dx		; save 'vgYend'
		mov	bx,[ds:vgColor]		; get color
		mov	[es:di+8],bx		; save color
		add	[ds:vgVecOfs],VECTMEMSZ	; point to next vector

 ifdef COUNT_VECTORS
		inc	[ds:_vectCount]		; increment count
 endif
		ret

		endp

;----------------------------------------------------------------------------;
; vgSwap
;
; Clear next screen to be displayed, draw all vectors, swap screens.
;----------------------------------------------------------------------------;
		align	4
public		vgSwap
proc		vgSwap	near

 if CLRSCN
		call	[ds:vSetErase]		; set VGA to erase
		call	[ds:vCls]		; clear current window
		mov	[ds:vgCurEraOfs],0	; reset pointer
		
 else
		call	[ds:vSetErase]		; set VGA to erase
		les	bp,[ds:vgCurEraPtr]	; point to end of array
		mov	[Byte es:bp+1],0FFh	; indicate end of array
		sub	bp,bp

vgClear:	mov	bx,[es:bp]		; get X start
		test	bh,bh			; end of array?
		js	vgClear2
		mov	cx,[es:bp+2]		; get Y start
		test	ch,ch
		js	vgClear1		; line was clipped, ignore
		mov	si,[es:bp+4]		; get X end
		mov	di,[es:bp+6]		; get Y end
		call	[ds:vLine]		; erase line
vgClear1:	add	bp,ERASMEMSZ		; point to next vector
		mov	es,[ds:vgCurEraSeg]	; get segment pointer
		jmp	Short vgClear		; keep erasing

vgClear2:	mov	[ds:vgCurEraOfs],0	; reset pointer
 endif

		les	bp,[ds:vgVecPtr]	; point to end of array
		mov	[Byte es:bp+2],00Fh	; set end of array marker
		mov	[ds:vgVecOfs],0		; start at top of array

; Loop drawing vectors

nextVector:	les	bp,[ds:vgVecPtr]	; point to next array entry

		mov	dx,[es:bp+2]		; get 'vgYStart'
		mov	cl,dl			; get range byte

		and	cl,0Fh			; mask high nibble
		cmp	cl,0Fh			; end of vectors?
		je	swapSeg			; yes, time to swap

		sar	dx,4			; sign extend 'vgYStart'

		mov	bx,[es:bp]		; get 'vgXstart'
		shl	bx,4
		sar	bx,4			; sign extend 'vgXstart'

		mov	si,[es:bp+4]		; get 'vgXend'
		shl	si,4
		sar	si,4			; sign extend X end
		sub	si,bx			; subtract start
		sar	si,cl			; unshift
		add	si,bx			; add start

		mov	ax,[es:bp+8]		; get color
		mov	[ds:vgTmpColor],ax	; save color

		mov	di,[es:bp+6]		; get 'vgYend'
		shl	di,4			; sign extend 'vgYend'
		sar	di,4
		sub	di,dx			; subtract start
		sar	di,cl			; unshift
		add	di,dx			; add start

 ifdef DEBUGGER
		mov	[ds:_vgRange],cl
		mov	[ds:_vgXstart],bx
		mov	[ds:_vgYstart],dx
		mov	[ds:_vgXend],si
		mov	[ds:_vgYend],di
 endif

; Check for a POINT, set brightness accordingly

		cmp	bx,si			; dot?
		jne	clip			; no, skip
		cmp	dx,di			; dot?
		jne	clip			; no, skip

; If a POINT, don't have to do as much.

		cmp	cl,[ds:vgTwinkle]	; compare range with twinkle, should be dim?
		jae	vgPnt1			; yes, use
		mov	ax,[ds:vBriColor]	; get bright color
		mov	[ds:vgTmpColor],ax	; make bright

vgPnt1:		mov	cx,dx			; get Y start in CX-reg

		cmp	bx,[ds:vgCineXmax]	; bigger than max?
		jg	clipXSR			; yes, clip X start right
		cmp	bx,[ds:vgCineXmin]	; less than min?
		jl	clipXSL			; yes, clip X start left
		sub	bx,[ds:vgCineXmin]

		cmp	cx,[ds:vgCineYmax]	; bigger than max?
		jg	clipYST			; yes, clip Y start top
		cmp	cx,[ds:vgCineYmin]	; less than min?
		jl	clipYSB			; yes, clip Y start bottom
		sub	cx,[ds:vgCineYmin]

		cmp	[ds:vgRotate],0		; rotate display?
		je	vgPnt2			; no, skip
		xchg	bx,cx			; exchange axis

vgPnt2:		mov	ax,cx
		mul	[ds:vgVgaYScale]	; scale to VGA value
		mov	ax,cx			; get position
		and	ax,[ds:vgVgaYMulOne]	; multiply by one?
		add	dx,ax			; if so, add original
		add	dx,[ds:vgVgaYOffset]	; add offset and flip
		xor	dx,[ds:vgVgaYSignFix]	; correct if negative
		mov	cx,dx			; save new Y VGA position

		mov	ax,bx
		mul	[ds:vgVgaXScale]	; scale to VGA value
		mov	ax,bx			; get position
		and	ax,[ds:vgVgaXMulOne]	; multiply by one?
		add	dx,ax			; if so, add original
		add	dx,[ds:vgVgaXOffset]	; add offset and flip
		xor	dx,[ds:vgVgaXSignFix]	; correct if negative
		mov	bx,dx			; save new X VGA position

		mov	ax,[ds:vgTmpColor]	; get color

 ifdef GRAPHICS

		; Save vectors in erase array

		les	bp,[ds:vgCurEraPtr]
		mov	[es:bp],bx		; save to erase VGA X start
		mov	[es:bp+2],cx		; save to erase VGA Y start
		mov	[es:bp+4],bx		; save to erase VGA X end
		mov	[es:bp+6],cx		; save to erase VGA Y end
		add	[ds:vgCurEraOfs],ERASMEMSZ ; increment pointer

		call	[ds:vSetColor]
		call	[ds:vPoint]
 endif
		add	[ds:vgVecOfs],VECTMEMSZ	; point to next vector
		jmp	nextVector		; draw next vector


; Clip line

clip:		mov	cx,dx			; get Y start in CX-reg

		cmp	bx,[ds:vgCineXmax]	; bigger than max?
		jg	clipXSR			; yes, clip X start right
		cmp	bx,[ds:vgCineXmin]	; less than min?
		jl	clipXSL			; yes, clip X start left
		sub	bx,[ds:vgCineXmin]

		cmp	cx,[ds:vgCineYmax]	; bigger than max?
		jg	clipYST			; yes, clip Y start top
		cmp	cx,[ds:vgCineYmin]	; less than min?
		jl	clipYSB			; yes, clip Y start bottom
		sub	cx,[ds:vgCineYmin]

		cmp	si,[ds:vgCineXmax]	; bigger than max?
		jg	clipXER			; yes, clip X end right
		cmp	si,[ds:vgCineXmin]	; less than min?
		jl	clipXEL			; yes, clip X end left
		sub	si,[ds:vgCineXmin]

		cmp	di,[ds:vgCineYmax]	; bigger than max?
		jg	clipYET			; yes, clip Y end top
		cmp	di,[ds:vgCineYmin]	; less than min?
		jl	clipYEB			; yes, clip Y end bottom
		sub	di,[ds:vgCineYmin]

; Scale points down to fit VGA screen

		cmp	[ds:vgRotate],0		; rotate display?
		je	clip1			; no, skip
		xchg	bx,cx			; exchange axis
		xchg	si,di

clip1:		mov	ax,cx
		mul	[ds:vgVgaYScale]	; scale to VGA value
		mov	ax,cx			; get position
		and	ax,[ds:vgVgaYMulOne]	; multiply by one?
		add	dx,ax			; if so, add original
		add	dx,[ds:vgVgaYOffset]	; add offset and flip
		xor	dx,[ds:vgVgaYSignFix]	; correct if negative
		mov	cx,dx			; save new Y VGA position

		mov	ax,bx
		mul	[ds:vgVgaXScale]	; scale to VGA value
		mov	ax,bx			; get position
		and	ax,[ds:vgVgaXMulOne]	; multiply by one?
		add	dx,ax			; if so, add original
		add	dx,[ds:vgVgaXOffset]	; add offset and flip
		xor	dx,[ds:vgVgaXSignFix]	; correct if negative
		mov	bx,dx			; save new X VGA position

		mov	ax,di			; move new Y to old
		mul	[ds:vgVgaYScale]	; scale to VGA value
		mov	ax,di			; get position
		and	ax,[ds:vgVgaYMulOne]	; multiply by one?
		add	dx,ax			; if so, add original
		add	dx,[ds:vgVgaYOffset]	; add offset and flip
		xor	dx,[ds:vgVgaYSignFix]	; correct if negative
		mov	di,dx			; save new Y VGA position

		mov	ax,si
		mul	[ds:vgVgaXScale]	; scale to VGA value
		mov	ax,si			; get position
		and	ax,[ds:vgVgaXMulOne]	; multiply by one?
		add	dx,ax			; if so, add original
		add	dx,[ds:vgVgaXOffset]	; add offset and flip
		xor	dx,[ds:vgVgaXSignFix]	; correct if negative
		mov	si,dx			; save new X VGA position

		mov	ax,[ds:vgTmpColor]	; get color

 ifdef GRAPHICS
		les	bp,[ds:vgCurEraPtr]
		mov	[es:bp],bx		; save to erase VGA X start
		mov	[es:bp+2],cx		; save to erase VGA Y start
		mov	[es:bp+4],si		; save to erase VGA X end
		mov	[es:bp+6],di		; save to erase VGA Y end
		add	[ds:vgCurEraOfs],ERASMEMSZ ; increment pointer

		call	[ds:vSetColor]
		call	[ds:vLine]
 endif

clipXSR:
clipXSL:
clipYST:
clipYSB:
clipXER:
clipXEL:
clipYET:
clipYEB:
		add	[ds:vgVecOfs],VECTMEMSZ	; point to next vector
		jmp	nextVector		; draw next vector

; Swap buffers

swapSeg:

 if 0
waitRetra1:	mov	dx,VGAINS1		; Input Status Reg #1
		in	al,dx
		test	al,008h			; Vertical retrace?
		jnz	waitRetra1		; no, loop

waitRetra2:	mov	dx,VGAINS1		; Input Status Reg #1
		in	al,dx
		test	al,008h			; Vertical retrace?
		jz	waitRetra2		; no, loop
 endif

		call	vSwap			; swap video pages

		mov	[ds:vgVecOfs],0		; reset vector pointer

		mov	eax,[ds:vgCurEraPtr]	; get current pointer
		xchg	[ds:vgLastEraPtr],eax	; xchg with last one
		mov	[ds:vgCurEraPtr],eax	; save new one

 ifdef COUNT_VECTORS
		mov	ax,[ds:_vectCount]
		cmp	ax,[ds:_vectMax]
		jbe	ttt1
		mov	[ds:_vectMax],ax
ttt1:		mov	[ds:_vectCount],0
 endif

		ret

		endp

		ends
		end
