;----------------------------------------------------------------------------;
; ZVGA.INC
;
; Definitions used to access VGA registers.
;
; Author:       Zonn Moore
; Last Updated: 02/04/00
;
; (c) Copyright 2000, Zonn Moore.  All rights reserved.
;----------------------------------------------------------------------------;
MODE		equ	10h	; Current display mode

VGAX		equ	640	; Number of pixels in X
VGAY		equ	350	; Number of pixels in Y
VGASIZE		equ	((640 / 8) * 350)

VGASEG1		equ	0A000h	; 1st segment of VGA memory
VGASEG2		equ	0A800h	; 2nd segment of VGA memory

VGAOFS1		equ	0
VGAOFS2		equ	VGASIZE

VGAAAD		equ	03C0h	; Address of Attribute Address Register
VGASEQ		equ	03C4h	; Address of Sequence register
VGACRT		equ	03D4h	; CRT control registers
VGAGCTL		equ	03CEh	; Address of VGA graphic control register
VGAPELR		equ	03C7h	; Address of PEL read address
VGAPELW		equ	03C8h	; Address of PEL write address
VGAPELD		equ	03C9h	; Address of PEL data Register
VGAINS1		equ	03DAh	; Input status register #1

; Graphic control register

vgrSR		equ	00h	; Set/Reset register
vgrESR		equ	01h	; Enable Set/Reset register
vgrCC		equ	02h	; Color Compare register
vgrDR		equ	03h	; Data Rotate register
vgrRMS		equ	04h	; Read Map Select register
vgrMODE		equ	05h	; Mode register
vgrMISC		equ	06h	; Miscellaneous register
vgrDCARE	equ	07h	; Don't care register
vgrBM		equ	08h	; Bit Mask register

; Sequence register

sqrCM		equ	01h	; Clocking Mode register

; CRT registers

crtSAH		equ	0Ch	; High byte of start address
crtSAL		equ	0Dh	; Low byte of start address

;----------------------------------------------------------------------------;
; SVGREG reg,val							     ;
;									     ;
; Set video register macro.						     ;
;									     ;
; Where:								     ;
;    reg = One of above vrXXX values					     ;
;    val = Value to set 'reg' (Can't be AL-reg)				     ;
;									     ;
; Assumes:								     ;
;    DX = Pointing to graphic control register of VGA			     ;
;									     ;
; Uses:									     ;
;    AL									     ;
;----------------------------------------------------------------------------;
macro		SVGREG	reg,val

		mov	al,reg
		out	dx,al
		inc	dx
		mov	al,val
		out	dx,al
		dec	dx

		endm

macro		SVGRAH	reg

		mov	al,reg
		out	dx,al
		inc	dx
		mov	al,ah
		out	dx,al
		dec	dx

		endm

;----------------------------------------------------------------------------;
; IOWAIT								     ;
;									     ;
; Macro for wasting time after an I/O access.				     ;
;----------------------------------------------------------------------------;
macro		IOWAIT
		local	carry,nocarry
		%NOCREF	carry,nocarry
		jc	carry	;; give the appearence of real code to any
		jnc	nocarry	;; intelligent instruction chacheing processor.
carry:		stc
nocarry:
		endm
